import pytest
from muppet.puppet.strings import (
    DocStringTag,
    DocString,
    DataTypeAlias,
)


def test_deserialize_from_json():
    tagname = 'Tagname'
    text = 'Contents of tagname'
    types = ['what even is this?']

    assert DocStringTag(tag_name=tagname, text=text, types=types) \
        == DocStringTag.from_json({
            'tag_name': tagname,
            'text': text,
            'types': types
        })


def test_deserialize_from_json_unknown_key():
    # TODO ensure that a worning is logged
    DocStringTag.from_json({
        'tag_name': 'Tagname',
        'text': 'Contents of tagname',
        'XXX': 'what even is this?'
    })


def test_deserialize_nested():
    text = 'A'
    tn = 'B'
    ttext = 'C'

    assert DocString(text=text,
                     tags=[DocStringTag(tag_name=tn,
                                        text=ttext)]) \
        == DocString.from_json({
            'text': text,
            'tags': [
                {
                    'tag_name': tn,
                    'text': ttext,
                    }
                ]
            })


def test_deserialize_from_json_type_error():
    with pytest.raises(TypeError):
        DocString.from_json({
            'text': 'text',
            'tags': [
                'no',
            ]
        })
