"""Tests for Parser combinator re-parsers."""

from muppet.puppet.format.parser import ParserFormatter
from muppet.puppet.ast import build_ast
from muppet.puppet.parser import puppet_parser
from muppet.parser_combinator import ParserCombinator


def test_parse_else_if():
    s = """
    if x {
      1
    } else {
      if y {
        2
      } else {
        3
      }
    }
    """
    ast = build_ast(puppet_parser(s))
    parser = ParserFormatter(s, "s").serialize(ast)
    match_objects = ParserCombinator(s, "s").get(parser)
    print(match_objects)
    # [`ws(['\n    '])`, `keyword(['if'])`, `ws([' '])`,
    # `qn([`ws([])`, 'x'])`, `ws([' '])`, '{', `ws(['\n      '])`,
    # `ws([])`, `ws([])`, '1', `ws(['\n    '])`, '}', `ws([' '])`,
    # `keyword(['else'])`, `ws([' '])`, '{', `ws(['\n      '])`,
    # `ws([])`, `keyword(['if'])`, `ws([' '])`, `qn([`ws([])`, 'y'])`,
    # `ws([' '])`, '{', `ws(['\n        '])`, `ws([])`, `ws([])`, '2',
    # `ws(['\n      '])`, '}', `ws([' '])`, `keyword(['else'])`,
    # `ws([' '])`, '{', `ws(['\n        '])`, `ws([])`, `ws([])`, '3',
    # `ws(['\n      '])`, '}', `ws(['\n    '])`, '}']


def test_parse_elsif():
    s = """
    if x {
      1
    } elsif y {
      2
    } else {
      3
    }
    """
    ast = build_ast(puppet_parser(s))
    parser = ParserFormatter(s, "s").serialize(ast)
    match_objects = ParserCombinator(s, "s").get(parser)
    print(match_objects)
    # [`ws(['\n    '])`, `keyword(['if'])`, `ws([' '])`,
    # `qn([`ws([])`, 'x'])`, `ws([' '])`, '{', `ws(['\n      '])`,
    # `ws([])`, `ws([])`, '1', `ws(['\n    '])`, '}', `ws([' '])`,
    # `keyword(['elsif'])`, `ws([' '])`, `qn([`ws([])`, 'y'])`, `ws(['
    # '])`, '{', `ws(['\n      '])`, `ws([])`, `ws([])`, '2', `ws(['\n
    # '])`, '}', `ws([' '])`, `keyword(['else'])`, `ws([' '])`, '{',
    # `ws(['\n      '])`, `ws([])`, `ws([])`, '3', `ws(['\n    '])`,
    # '}']


from pprint import pprint


def test_chained():
    s = "x.filter().join()"
    ast = build_ast(puppet_parser(s))
    pprint(ast)
    parser = ParserFormatter(s, "s").serialize(ast)
    pprint(parser)
    match_objects = ParserCombinator(s, "s").get(parser)
    pprint(match_objects)


def test_lambda():
    s = "x.filter |$x, $y| { $x + 1 }"
    ast = build_ast(puppet_parser(s))
    pprint(ast)
    parser = ParserFormatter(s, "s").serialize(ast)
    pprint(parser)
    match_objects = ParserCombinator(s, "s").get(parser)
    pprint(match_objects)


def test_chained_and_lambda():
    s = "x.filter |$x, $y| { $x + 1 }.join()"
    ast = build_ast(puppet_parser(s))
    pprint(ast)
    parser = ParserFormatter(s, "s").serialize(ast)
    print("parser:\n" + str(parser))
    match_objects = ParserCombinator(s, "s").get(parser)
    pprint(match_objects)
